*! Version 1.0
*! Version 1.1, converts string vars to numeric via "encode"
*! Version 1.2, adds missing() option, with -12345 as default missing code
*!              Handles version 8 missing codes .a etc.
*! Version 1.2.1 Added message about sorting data
capture program drop stata2mlwin
program define stata2mlwin
  version 7
  syntax using/ , [ replace clear nocons missing(integer -12345 ) ]
  
  use "`using'" , `clear'
  if "`nocons'" == "" {
    capture local hascons : type cons
    if _rc != 0 {
      gen cons = 1
      order cons
    }
    else {
      display as error "Variable cons already exists, no constant created"
      display "Conversion continues"
    }
  }

  tempvar out
  capture file close `out'

  quietly file open `out' using "`using'.obe", write text `replace'

  unab varlist : *
  local varcnt: word count `varlist'
  * display " varlist is `varlist' has `varcnt'"

  foreach var of local varlist {
    local vartype : type `var' 
    if (substr("`vartype'",1,3)=="str") {
      display "encoding `var'"
      tempvar tempenc
      encode `var', generate(`tempenc')
      drop `var'
      rename `tempenc' `var'
    }
  }

  display "Missing values stored as `missing' 
  display "(You can change this with the missing() option.)"
  display "In MLwiN, choose Options Worksheet then Numbers"
  display "to tell it that values that are `missing' are missing."

  quietly count
  local ncases = `r(N)'

  * file write `out' "wipe" _newline
  file write `out' "echo 0" _newline
  file write `out' "assign c1-c`varcnt'" _newline
  
  local i = 1
  while (`i' <= `ncases') {
    if mod(`i' , 100 ) == 0 {
      display _column(1) "." _continue
    }
    foreach var of local varlist {
      local vartype : type `var' 
      if (("`vartype'"=="byte") | ("`vartype'"=="int")  | ("`vartype'"== "long")  | ("`vartype'"== "float") | ("`vartype'"== "double")) {
        if (`var'[`i'] >= .) {
          file write `out' "`missing'" " "
        }
        else {
          file write `out' (`var'[`i']) " "
        }
      }
      else {
        file write `out' (`var'[`i']) " "
      }
    }
    file write `out' _newline
    local i = `i' + 1
  } 
  display

  file write `out' "finish" _newline
  file write `out' "echo 1" _newline

  file write `out' _newline
  local i = 0
  foreach var of local varlist {
    local i = `i' + 1
    file write `out' "name c`i' '`var''" _newline
  }

  file write `out' _newline
  local i = 0
  foreach var of local varlist {
    local i = `i' + 1
    makelab `var' `i' `ncases' `out'
  }

  local pwd : pwd
  * if index("`using'","\") == 0 {
  *   file write `out'  _newline "save `pwd'" "\" "`using'.ws" _newline
  * }
  * else {
  *   file write `out'  _newline "save `using'.ws" _newline
  * }

  file close `out'

  display as text "Looks like this was a success."
  display as text "To convert the file to MLwiN, start MLwiN and then"
  display as text "Choose 'Data Manipulation -> Command Interface" 
  display as text "then at the command prompt type"
  display as input "  wipe"
  display as text "which clears out any data (if any), then type"
  if index("`using'","\") == 0 {
    display as input "  obey `pwd'" "\" "`using'.obe"
  }
  else {
    display as input "  obey `using'.obe"
  }
  display as text "which runs the program to make the MLwiN data file."
  display as text "You can then choose 'File' -> 'Save as' to save the file."

  display
  display "NOTE: Please be sure your data is sorted based on the levels"
  display "of your model.  For example, for a 2 level model, based on the
  display "level 2 ID, then the level 1 ID.  You can do this in Stata before"
  display "running this program, or in MLwiN afterwards."

  clear

end

capture program drop makelab
program define makelab
  * variable name, variable number, n of cases, file handle
  args var i n out
  * display "args `var' `i' `n'"

  * CATN 0 c1
  * CATN 1 c1 0 '\male' 1 '\female'

  local vl : value label `var'
  if "`vl'" == "" {
    exit 0
  }

  tempvar lvar
  tempvar first
  decode `var', gen(`lvar')

  sort `lvar'
  by `lvar' : gen `first' = (_n == 1) & `lvar' != ""
 
  file write `out' "CATN 0 C`i'" _newline

  local casenum = 1
  while (`casenum' <= `n') {
    if `first'[`casenum'] {
      local v1 = `var'[`casenum']
      local v2 = `lvar'[`casenum']
      file write `out' "CATN 1 C`i' `v1' '\" "`v2'" "'" _newline
    }
    local casenum = `casenum' + 1
  } 
  file write `out' _newline
end
  


 

